<!-----------------------------------------------------------------------
Author 	 :	Luis Majano
Date     :	September 25, 2005
Description :

	The galleon administrator port.

----------------------------------------------------------------------->
<cfcomponent name="ehForums" extends="coldbox.system.eventhandler" output="false">

	<cffunction name="onAppStart" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<!--- App Init Code --->
		<cfset structDelete(application, "userCache")>
		
		<!--- Get main settings --->
		<cfinvoke component="#getSetting("ParentMapping")#/cfcs/galleon" method="getSettings" returnVariable="settings">
		<cfset application.settings = settings>
		
		<!--- Attachment Directory --->
		<cfset application.settings.attachmentdir = expandPath("../attachments")>
		<cfif not directoryExists(application.settings.attachmentdir)>
			<cfdirectory action="create" directory="#application.settings.attachmentdir#">
		</cfif>
		
		<!--- get user CFC --->
		<cfset application.user = createObject("component","#getSetting("ParentMapping")#.cfcs.user").init(application.settings)>
		<!--- get utils CFC --->
		<cfset application.utils = createObject("component","#getSetting("ParentMapping")#.cfcs.utils")>
		<!--- get conference CFC --->
		<cfset application.conference = createObject("component","#getSetting("ParentMapping")#.cfcs.conference").init(application.settings)>
		<!--- get forum CFC --->
		<cfset application.forum = createObject("component","#getSetting("ParentMapping")#.cfcs.forum").init(application.settings)>
		<!--- get thread CFC --->
		<cfset application.thread = createObject("component","#getSetting("ParentMapping")#.cfcs.thread").init(application.settings)>
		<!--- get message CFC --->
		<cfset application.message = createObject("component","#getSetting("ParentMapping")#.cfcs.message").init(application.settings)>
		<!--- get rank CFC --->
		<cfset application.rank = createObject("component","#getSetting("ParentMapping")#.cfcs.rank").init(application.settings)>
		<cfset application.init = true>
	</cffunction>
	
	<cffunction name="onRequestStart" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		<!--- Logout Code --->
		<cfif Event.valueExists("logout")>
			<cfset doLogout()>
		</cfif>

		<!--- handle security --->
		<cflogin></cflogin>

		<!--- Used by index, forums, and threads ---->
		<cfset Event.paramValue("sort", "")>
		
		<cfif not isLoggedOn() and not findnocase("Login", Event.getValue("event"))>
			<cfset Event.setValue("event","ehForums.dspLogin")>
		</cfif>
		
		<!--- must be the correct authentication --->
		<cfif not isUserInRole("forumsadmin") and not findnocase("login", Event.getValue("event"))>
			<cfset getPlugin("messagebox").setMessage("warning","Invalid Role detected.")>
			<cfset Event.setValue("event","ehForums.dspLogin")>
		</cfif>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehConferences = "ehforums.dspConferences">
		<cfset rc.xehForums = "ehforums.dspForums">
		<cfset rc.xehThreads = "ehforums.dspThreads">
		<cfset rc.xehMessages = "ehforums.dspMessages">
		<cfset rc.xehRanks = "ehforums.dspRanks">
		<cfset rc.xehUsers = "ehforums.dspUsers">
		<cfset rc.xehStats = "ehforums.dspStats">
		<cfset rc.xehSearchStats = "ehForums.dspSearchStats">
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspHome" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Welcome to the Galleon Administrator")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwIndex")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- SECURITY HANDLERS --->
	<!--- ************************************************************* --->
	<cffunction name="dspLogin" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehLogin = "ehForums.doLogin">
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwLogin")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doLogin" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var mygroups = "">
		
		<!--- handle security --->
		<cflogin >
			<cfif application.user.authenticate(trim(Event.getValue("username")), trim(Event.getValue("password")))>
				<!--- good logon, grab their groups --->
				<cfset mygroups = application.user.getGroupsForUser(trim(Event.getValue("username")))>
				<cfset session.user = application.user.getUser(trim(Event.getValue("username")))>
				<cfloginuser name="#trim(Event.getValue("username"))#" password="#trim(Event.getValue("password"))#" roles="#mygroups#">
			<cfelse>
				<cfset getPlugin("messagebox").setMessage("error","Your username and password did not work.")>
				<cfset setNextEvent("ehForums.dspLogin","failedLogon=true&ref=#Event.getValue("ref","")#")>
			</cfif>
		</cflogin>
		<!-- Set NExt Event --->
		<cfset setNextEvent("ehForums.dspHome")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doLogout" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<!--- handle security --->
		<cflogout>
		<cfset setNextEvent("ehForums.dspLogin")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- CONFERENCES --->
	<!--- ************************************************************* --->
	<cffunction name="dspConferences" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehConferencesEdit = "ehForums.dspConferencesEdit">
		
		<!--- handle deletions --->
		<cfif Event.valueExists("mark") and len(Event.getValue("mark"))>
			<cfset doDeleteConference(Event)>
		</cfif>

		<!--- get conferences --->
		<cfset Event.setValue("conferences", application.conference.getConferences(false))>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Conference Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwConferences")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteConference" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var id = "">
		<cfloop index="id" list="#Event.getValue("mark")#">
			<cfset application.conference.deleteConference(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Conferences deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspConferencesEdit" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var conference = "">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehSaveConference = "ehForums.doSaveConference">
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspConferences")>
		</cfif>

		<cfif Event.getValue("id") neq "0">
			<cfset conference = application.conference.getConference(Event.getValue("id")) >
			<cfset Event.setValue("name", conference.name)>
			<cfset Event.setValue("description", conference.description)>
			<cfset Event.setValue("active", conference.active)>
		<cfelse>
			<cfset Event.setValue("active", false)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Conference Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwConferences_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveConference" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var errors = "">
		<cfset var conference = "">
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspConferences")>
		</cfif>

		<cfif not len(trim(Event.getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("description")))>
			<cfset errors = errors & "You must specify a description.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset conference = structNew()>
			<cfset conference.name = trim(htmlEditFormat(Event.getValue("name")))>
			<cfset conference.description = trim(htmlEditFormat(Event.getValue("description")))>
			<cfset conference.active = trim(htmlEditFormat(Event.getValue("active")))>
			<cfif Event.getValue("id") neq 0>
				<cfset application.conference.saveConference(Event.getValue("id"), conference)>
			<cfelse>
				<cfset application.conference.addConference(conference)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Conferfence, #conference.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspConferences")>
		</cfif>
		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspConferencesEdit","id=#Event.getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- FORUMS --->
	<!--- ************************************************************* --->
	<cffunction name="dspForums" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehForumsEdit = "ehForums.dspForumsEdit">
		<!--- handle deletions --->
		<cfif Event.valueExists("mark") and len(Event.getValue("mark"))>
			<cfset doDeleteForums(Event)>
		</cfif>

		<!--- get forums --->
		<cfset Event.setValue("forums",application.forum.getForums(false) )>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Forum Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwForums")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteForums" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#Event.getValue("mark")#">
			<cfset application.forum.deleteForum(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Forums deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspForumsEdit" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var forum = "">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehForumsSave = "ehForums.doSaveForums">
		
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspForums")>
		</cfif>

		<!--- get all conferences --->
		<cfset Event.setValue("conferences", application.conference.getConferences(false))>

		<cfif Event.getValue("id") neq "0">
			<cfset forum = application.forum.getForum(Event.getValue("id"))>
			<cfset Event.setValue("name", forum.name)>
			<cfset Event.setValue("description", forum.description)>
			<cfset Event.setValue("readonly", forum.readonly)>
			<cfset Event.setValue("active", forum.active)>
			<cfset Event.setValue("conferenceidfk", forum.conferenceidfk)>
			<cfset Event.setValue("attachments",forum.attachments)>
		<cfelse>
			<cfset Event.setValue("readonly",false)>
			<cfset Event.setValue("active", false)>
			<cfset Event.setValue("attachments","false")>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Forum Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwForums_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveForums" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var errors = "">
		<cfset var forum = "">
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspForums")>
		</cfif>
		
		<cfif not len(trim(Event.getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("description")))>
			<cfset errors = errors & "You must specify a description.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset forum = structNew()>
			<cfset forum.name = trim(htmlEditFormat(Event.getValue("name")))>
			<cfset forum.description = trim(htmlEditFormat(Event.getValue("description")))>
			<cfset forum.readonly = trim(htmlEditFormat(Event.getValue("readonly")))>
			<cfset forum.active = trim(htmlEditFormat(Event.getValue("active")))>
			<cfset forum.attachments = trim(htmlEditFormat(Event.getValue("attachments")))>
			<cfset forum.conferenceidfk = trim(htmlEditFormat(Event.getValue("conferenceidfk")))>

			<cfif Event.getValue("id") neq 0>
				<cfset application.forum.saveForum(Event.getValue("id"), forum)>
			<cfelse>
				<cfset application.forum.addForum(forum)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Forum, #forum.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspForums")>
		</cfif>

		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspForumsEdit","id=#Event.getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->


	<!--- THREADS --->
	<!--- ************************************************************* --->
	<cffunction name="dspThreads" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehThreadsEdit = "ehForums.dspThreadsEdit">
		
		<!--- handle deletions --->
		<cfif Event.valueExists("mark") and len(Event.getValue("mark"))>
			<cfset doDeleteThreads(Event)>
		</cfif>

		<!--- get Threads --->
		<cfset Event.setValue("threads",application.thread.getThreads(false) )>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Thread Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwThreads")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteThreads" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#Event.getValue("mark")#">
			<cfset application.thread.deleteThread(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Threads deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspThreadsEdit" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		<cfset var thread = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehThreadsSave = "ehForums.doSaveThreads">
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspThreads")>
		</cfif>

		<!--- get all forums --->
		<cfset Event.setValue("forums", application.forum.getForums(false) )>
		<!--- get all users --->
		<cfset Event.setValue("users", application.user.getUsers() )>

		<cfif Event.getValue("id") neq "0">
			<cfset thread = application.thread.getThread(Event.getValue("id"))>
			<cfset Event.setValue("name", thread.name)>
			<cfset Event.setValue("readonly", thread.readonly)>
			<cfset Event.setValue("active", thread.active)>
			<cfset Event.setValue("forumidfk", thread.forumidfk)>
			<cfset Event.setValue("datecreated", dateFormat(thread.datecreated,"m/dd/yy"))>
			<cfset Event.setValue("useridfk", thread.useridfk)>
			<cfset Event.setValue("sticky", thread.sticky)>
		<cfelse>
			<cfset Event.setValue("readonly", false)>
			<cfset Event.setValue("active", false)>
			<cfset Event.setValue("datecreated", dateFormat(now(),"m/dd/yy"))>
			<cfset Event.setValue("sticky", false)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Conference Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwThreads_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveThreads" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var errors = "">
		<cfset var thread = "">
		
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspThreads")>
		</cfif>
		
		<cfif not len(trim(Event.getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("datecreated"))) or not isDate(Event.getValue("datecreated"))>
			<cfset errors = errors & "You must specify a valid creation date.<br>">
		</cfif>

		<cfif not len(errors)>
			<cfset thread = structNew()>
			<cfset thread.name = trim(htmlEditFormat(Event.getValue("name")))>
			<cfset thread.readonly = trim(htmlEditFormat(Event.getValue("readonly")))>
			<cfset thread.active = trim(htmlEditFormat(Event.getValue("active")))>
			<cfset thread.forumidfk = trim(htmlEditFormat(Event.getValue("forumidfk")))>
			<cfset thread.datecreated = trim(htmlEditFormat(Event.getValue("datecreated")))>
			<cfset thread.useridfk = trim(htmlEditFormat(Event.getValue("useridfk")))>
			<cfset thread.sticky = trim(htmlEditFormat(Event.getValue("sticky")))>
			<cfif Event.getValue("id") neq 0>
				<cfset application.thread.saveThread(Event.getValue("id"), thread)>
			<cfelse>
				<cfset application.thread.addThread(thread)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Thread, #thread.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspThreads")>
		</cfif>

		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspThreadsEdit","id=#Event.getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->


	<!--- MESSAGES --->
	<!--- ************************************************************* --->
	<cffunction name="dspMessages" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehMessagesEdit = "ehForums.dspMessagesEdit">
		<!--- handle deletions --->
		<cfif Event.valueExists("mark") and len(Event.getValue("mark"))>
			<cfset doDeleteMessages(Event)>
		</cfif>

		<!--- get Threads --->
		<cfset Event.setValue("messages",application.message.getMessages() )>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Messages Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwMessages")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteMessages" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#Event.getValue("mark")#">
			<cfset application.message.deleteMessage(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Messages deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspMessagesEdit" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		<cfset var message = "">
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehMessagesSave = "ehForums.doSaveMessages">
		<cfset rc.xehAttachment = "ehForums.dspAttachment">
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspMessages")>
		</cfif>

		<!--- get all forums --->
		<cfset Event.setValue("threads", application.thread.getThreads(false) )>
		<!--- get all users --->
		<cfset Event.setValue("users", application.user.getUsers() )>

		<cfif Event.getValue("id") neq "0">
			<cfset message = application.message.getMessage(Event.getValue("id"))>
			<cfset Event.setValue("title", message.title)>
			<cfset Event.setValue("body", message.body)>
			<cfset Event.setValue("posted", dateFormat(message.posted,"m/dd/yy") & timeFormat(message.posted,"h:mm tt"))>
			<cfset Event.setValue("useridfk", message.useridfk)>
			<cfset Event.setValue("threadidfk", message.threadidfk)>
			<cfset Event.setValue("attachment", message.attachment)>
		<cfelse>
			<cfset Event.setValue("posted","#dateFormat(now(),"m/dd/yy")# #timeFormat(now(),"h:mm tt")#")>
			<cfset Event.setValue("title","")>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwMessages_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveMessages" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var errors = "">
		<cfset var message = "">
		<cfset var currentMessage = "">
		<cfset var users = "">
		<cfset var threadPicked = "">
		<cfset var theusername = "">
		<cfset var rc = Event.getCollection()>
		
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspMessages")>
		</cfif>

		<cfif not len(trim(Event.getValue("title")))>
			<cfset errors = errors & "You must specify a title.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("body")))>
			<cfset errors = errors & "You must specify a body.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("posted"))) or not isDate(Event.getValue("posted"))>
			<cfset errors = errors & "You must specify a valid creation date.<br>">
		</cfif>

		<cfif not len(errors)>
			
			<cfset message = structNew()>
			<cfif rc.id neq 0>
				<cfset currentMessage = application.message.getMessage(rc.id)>
			</cfif>
			<!--- Remove The File --->
			<cfif structKeyExists(rc, "removefile")>
				<cfif currentMessage.attachment neq "" and fileExists(application.settings.attachmentdir & getSetting("OSFileSeparator",1) & currentMessage.filename)>
					<cffile action="delete" file="#application.settings.attachmentdir#/#currentMessage.filename#">
					<cfset message.attachment = "">
					<cfset message.filename = "">
				</cfif>
			<cfelseif rc.id neq 0>
				<cfset message.attachment = currentMessage.attachment>
				<cfset message.filename = currentMessage.filename>
			<cfelse>
				<cfset message.attachment = "">
				<cfset message.filename = "">
			</cfif>
			
			<cfset message.title = trim(htmlEditFormat(Event.getValue("title")))>
			<cfset message.body = trim(htmlEditFormat(Event.getValue("body")))>
			<cfset message.posted = trim(Event.getValue("posted"))>
			<cfset message.threadidfk = Event.getValue("threadidfk")>
			<cfset message.useridfk = Event.getValue("useridfk")>

			<!--- get all users --->
			<cfset users = application.user.getUsers() >

			<cfif Event.getValue("id") neq 0>
				<cfset application.message.saveMessage(Event.getValue("id"), message)>
			<cfelse>
				<cfset threadPicked = application.thread.getThread(Event.getValue("threadidfk"))>
				<!--- translate the user id to the username for addMessage --->
				<cfloop query="users">
					<cfif Event.getValue("id") is Event.getValue("useridfk")>
						<cfset theusername = username>
					</cfif>
				</cfloop>
				<cfset application.message.addMessage(message,threadPicked.forumidfk,theusername,Event.getValue("threadidfk"))>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Message, #message.title#, has been updated.")>
			<cfset setNextEvent("ehForums.dspMessages")>
		</cfif>
		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspMessagesEdit","id=#Event.getValue("id")#")>
	</cffunction>
	<!--- ************************************************************* --->


	<!--- RANKS --->
	<!--- ************************************************************* --->
	<cffunction name="dspRanks" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehRanksEdits = "ehForums.dspRanksEdit">
		<!--- handle deletions --->
		<cfif Event.valueExists("mark") and len(Event.getValue("mark"))>
			<cfset doDeleteRanks(Event)>
		</cfif>

		<!--- get Ranks --->
		<cfset Event.setValue("ranks",application.rank.getRanks() )>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Rank Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwRanks")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteRanks" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#Event.getValue("mark")#">
			<cfset application.rank.deleteRank(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Ranks deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspRanksEdit" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rank = "">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehRanksSave = "ehForums.doSaveRanks">
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspRanks")>
		</cfif>

		<cfif Event.getValue("id") neq "0">
			<cfset rank = application.rank.getRank(Event.getValue("id"))>
			<cfset Event.setValue("name", rank.name)>
			<cfset Event.setValue("minposts", rank.minposts)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Conference Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwRanks_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveRanks" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var errors = "">
		<cfset var rank = "">
		
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspRanks")>
		</cfif>

		<cfif not len(trim(Event.getValue("name")))>
			<cfset errors = errors & "You must specify a name.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("minposts"))) or not isNumeric(Event.getValue("minposts")) or Event.getValue("minposts") lt 0>
			<cfset errors = errors & "You must specify a numeric value greater than zero for the minimum number of posts.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset rank = structNew()>
			<cfset rank.name = trim(htmlEditFormat(Event.getValue("name")))>
			<cfset rank.minposts = trim(htmlEditFormat(Event.getValue("minposts")))>
			<cfif Event.getValue("id") neq 0>
				<cfset application.rank.saveRank(Event.getValue("id"), rank)>
			<cfelse>
				<cfset application.rank.addRank(rank)>
			</cfif>
			<!--- MessageBox --->
			<cfset getPlugin("messagebox").setMessage("info","Rank, #rank.name#, has been updated.")>
			<cfset setNextEvent("ehForums.dspRanks")>
		</cfif>

		<!--- Error messages --->
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspRanksEdit","id=#Event.getValue("id")#")>

	</cffunction>
	<!--- ************************************************************* --->

	<!--- USERS --->
	<!--- ************************************************************* --->
	<cffunction name="dspUsers" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehUsersEdit = "ehForums.dspUsersEdit">
		<!--- handle deletions --->
		<cfif Event.valueExists("mark") and len(Event.getValue("mark"))>
			<cfset doDeleteUsers(Event)>
		</cfif>

		<!--- get Ranks --->
		<cfset Event.setValue("users",application.user.getUsers() )>
		
		<cfif application.settings.requireconfirmation>
			<cfset rc.list = "username,emailaddress,postcount,confirmed,datecreated">
		<cfelse>
			<cfset rc.list = "username,emailaddress,postcount,datecreated">
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Users Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwUsers")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doDeleteUsers" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var id = "">
		<!--- handle deletions --->
		<cfloop index="id" list="#Event.getValue("mark")#">
			<cfset application.user.deleteUser(id)>
		</cfloop>
		<!--- message box --->
		<cfset getPlugin("messagebox").setMessage("info","Users deleted successfully")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspUsersEdit" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var user = "">
		<cfset var rc = Event.getCollection()>
		
		<!--- EXIT HANDLERS: --->
		<cfset rc.xehUsersSave = "ehForums.doSaveUsers">
		
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspUsers")>
		</cfif>

		<cfset Event.setValue("qGroups",application.user.getGroups())>

		<cfif Event.getValue("id") gte "1">
			<cfset user = application.user.getUser(Event.getValue("id"))>
			<cfset Event.setValue("username", user.username)>
			<cfset Event.setValue("emailaddress", user.emailaddress)>
			<cfset Event.setValue("password", user.password)>
			<cfset Event.setValue("datecreated", dateFormat(user.datecreated,"m/dd/yy") & timeFormat(user.datecreated,"h:mm tt"))>
			<cfset Event.setValue("groups", user.groups)>
			<cfset Event.setValue("confirmed", user.confirmed)>
		<cfelse>
			<cfset Event.setValue("datecreated",dateFormat(now(),"m/dd/yy") & timeFormat(now(),"h:mm tt") )>
			<cfset Event.setValue("confirmed","false")>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Conference Editor")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwUsers_edit")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doSaveUsers" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		<cfset var errors = "">
		<cfset var user = "">
		
		<!--- Checks --->
		<cfif Event.valueExists("cancel") or not Event.valueExists("id") or not len(Event.getValue("id"))>
			<cfset setNextEvent("ehForums.dspUsers")>
		</cfif>
		
		<cfif not len(trim(Event.getValue("username")))>
			<cfset errors = errors & "You must specify a username.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("emailaddress"))) or not isEmail(trim(Event.getValue("emailaddress")))>
			<cfset errors = errors & "You must specify an emailaddress.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("password")))>
			<cfset errors = errors & "You must specify a password.<br>">
		</cfif>
		<cfif not len(trim(Event.getValue("datecreated"))) or not isDate(Event.getValue("datecreated"))>
			<cfset errors = errors & "You must specify a creation date.<br>">
		</cfif>
		<cfif not structKeyExists(rc,"groups") or not len(rc.groups)>
			<cfset errors = errors & "You must specify at least one group for the user.<br>">
		</cfif>
		<cfif not len(errors)>
			<cfset user = structNew()>
			<cfset Event.setValue("username", trim(htmlEditFormat(Event.getValue("username"))))>
			<cfset Event.setValue("emailaddress", trim(htmlEditFormat(Event.getValue("emailaddress"))))>
			<cfset Event.setValue("password",trim(htmlEditFormat(Event.getValue("password"))))>
			<cfset Event.setValue("datecreated", trim(htmlEditFormat(Event.getValue("datecreated"))))>
			<cfset Event.paramValue("groups","")>
			<cfset Event.paramValue("confirmed","true")>
			<cfif Event.getValue("id") neq 0>
				<cfset application.user.saveUser(Event.getValue("username"), Event.getValue("password"), Event.getValue("emailaddress"), Event.getValue("datecreated"),Event.getValue("groups"),rc.confirmed)>
			<cfelse>
				<cftry>
					<cfset application.user.addUser(Event.getValue("username"), Event.getValue("password"), Event.getValue("emailaddress"), Event.getValue("groups"), rc.confirmed)>
					<cfcatch>
						<cfset errors = cfcatch.message>
					</cfcatch>
				</cftry>
			</cfif>
			<cfif not len(errors)>
				<!--- MessageBox --->
				<cfset getPlugin("messagebox").setMessage("info","User, #Event.getValue("username")#, has been updated.")>
				<cfset setNextEvent("ehForums.dspUsers")>
			</cfif>
		</cfif>
		<cfset getPlugin("messagebox").setMessage("error",errors)>
		<cfset setNextEvent("ehForums.dspUsersEdit","id=#Event.getValue("id")#")>

	</cffunction>
	<!--- ************************************************************* --->


	<!--- STATS --->
	<!--- ************************************************************* --->
	<cffunction name="dspStats" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		
		<cfset Event.setValue("charts", true)>
		<cfif server.coldfusion.productname is "BlueDragon">
			<cfset Event.setValue("charts", false)>
		</cfif>

		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Galleon Stats")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwStats")>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="doGetStats" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var messages = "">
		<!--- The framework has the ability to call and run an event handler from a tag --->
		<cfset Event.setValue("conferences", application.conference.getConferences())>
		<cfset Event.setValue("forums",application.forum.getForums())>
		<cfset Event.setValue("threads", application.thread.getThreads())>
		<cfset Event.setValue("users", application.user.getUsers())>
		<cfset messages = application.message.getMessages()>

		<!--- get first post --->
		<cfquery name="getMinPost" dbtype="query">
		select 	min(posted) as earliestPost
		from	messages
		</cfquery>
		<!--- get last post --->
		<cfquery name="getMaxPost" dbtype="query">
		select 	max(posted) as lastPost
		from	messages
		</cfquery>

		<!--- Save in Request --->
		<cfset Event.setValue("getMinPost", getMinPost)>
		<cfset Event.setValue("getMaxPost", getMaxPost)>
		<cfset Event.setValue("messages",messages)>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="dspSearchStats" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<cfset var rc = Event.getCollection()>
		<cfset var prefix = application.settings.tableprefix>
		<!--- will be used to generate general stats - so we get all from the last 365 days --->
		<cfset var oldest = dateAdd("d", -365, now())>
		<cfset var searchstats = "">
		<cfset var today = createDateTime(year(now()), month(now()), day(now()), 0,0,0)>
		<cfset var past30 = dateAdd("d", -30, now())>
		<cfset var past365 = dateAdd("d", -365, now())>
		
		<cfquery datasource="#application.settings.dsn#" name="searchstats">
			select	searchterms, datesearched
			from	#prefix#search_log
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_date" value="#oldest#">
		</cfquery>
		
		<!--- ineffecient - but gets around the lack of a limit op in ms access --->
		<cfquery name="rc.latest" dbtype="query" maxrows="1">
			select searchterms, datesearched
			from	searchstats
			order by datesearched desc
		</cfquery>
		
		<cfquery name="rc.todaysbest" dbtype="query" maxrows="1">
			select	searchterms, count(searchterms) as score
			from	searchstats
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_timestamp" value="#today#">
			group by searchterms
			order by score desc
		</cfquery>
		
		
		<cfquery name="rc.thismonth" dbtype="query" maxrows="1">
			select	searchterms, count(searchterms) as score
			from	searchstats
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_timestamp" value="#past30#">
			group by searchterms
			order by score desc
		</cfquery>
		
		<cfquery name="rc.thisyear" dbtype="query" maxrows="1">
			select	searchterms, count(searchterms) as score
			from	searchstats
			where	datesearched > <cfqueryparam cfsqltype="cf_sql_timestamp" value="#past365#">
			group by searchterms
			order by score desc
		</cfquery>

		<cfquery name="rc.top30" dbtype="query" maxrows="30">
			select	searchterms, count(searchterms) as score
			from	searchstats
			group by searchterms
			order by score desc
		</cfquery>


		<!--- Set Title and Templatename --->
		<cfset Event.setValue("title","Galleon Search Stats")>
		<cfset Event.setValue("templatename","admin")>
		<!--- Set the View To Display, after Logic --->
		<cfset Event.setView("vwSearch_stats")>
	</cffunction>
	<!--- ************************************************************* --->
	
	
	<cffunction name="dspAttachment" access="public" returntype="void" output="false">
		<cfargument name="Event" type="coldbox.system.beans.requestContext">
		<!---
		Name         : attachment.cfm
		Author       : Raymond Camden 
		Created      : November 3, 2004
		Last Updated : 
		History      : 
		Purpose		 : Load an attachment
	--->
		<cfset var message = "">
		<cfset var extension = "">
		<cfset var rc = Event.getCollection()>
		
		<cfif not Event.valueExists("id") or not len(rc.id)>
			<cfset setNextEvent("ehForums.dspHome")>
		</cfif>
		
		<cftry>
			<cfset message = application.message.getMessage(rc.id)>
			<cfcatch>
				<cfset getPlugin("logger").logError("Error getting Message", cfcatch)>
				<cfset setNextEvent()>
			</cfcatch>
		</cftry>
		
		<cfif not len(message.filename)>
			<cfset setNextEvent()>
		</cfif>
		
		<cfif not fileExists(application.settings.attachmentdir & getSetting("OSFileSeparator",1) & message.filename)>
			<cfset setNextEvent()>
		</cfif>
		
		<cfset extension = listLast(message.filename, ".")>
		
		<cfswitch expression="#extension#">
		
			<cfcase value="txt">
				<cfheader name="Content-disposition" value="attachment;filename=#message.filename#">
				<cfcontent file="#application.settings.attachmentdir#/#message.filename#" type="text/plain">
			</cfcase>
					
			<cfcase value="pdf">
				<cfheader name="Content-disposition" value="attachment;filename=#message.filename#">		
				<cfcontent file="#application.settings.attachmentdir#/#message.filename#" type="application/pdf">
			</cfcase>
				
			<cfcase value="doc">
				<cfheader name="Content-disposition" value="attachment;filename=#message.filename#">		
				<cfcontent file="#application.settings.attachmentdir#/#message.filename#" type="application/msword">		
			</cfcase>
			
			<cfcase value="ppt">
				<cfheader name="Content-disposition" value="attachment;filename=#message.filename#">		
				<cfcontent file="#application.settings.attachmentdir#/#message.filename#" type="application/vnd.ms-powerpoint">		
			</cfcase>
			
			<cfcase value="xls">
				<cfheader name="Content-disposition" value="attachment;filename=#message.filename#">		
				<cfcontent file="#application.settings.attachmentdir#/#message.filename#" type="application/application/vnd.ms-excel">		
			</cfcase>
		
			<cfcase value="zip">
				<cfheader name="Content-disposition" value="attachment;filename=#message.filename#">		
				<cfcontent file="#application.settings.attachmentdir#/#message.filename#" type="application/application/zip">		
			</cfcase>
			
			<!--- everything else --->
			<cfdefaultcase>
				<cfheader name="Content-disposition" value="attachment;filename=#message.filename#">		
				<cfcontent file="#application.settings.attachmentdir#/#message.filename#" type="application/unknown">		
			</cfdefaultcase>
					
		</cfswitch>
		
		<cfabort>
	</cffunction>

	</cfcomponent>